﻿using System;
using System.Globalization;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using BackgroundTaskManagerDemo;

namespace BackgroundTaskManagerDemoSL
{
    public partial class Page : UserControl
    {
        public Page()
        {
            InitializeComponent();
            PendingTaskCount = 0;

            // Hook up to BackgroundTaskManager to track status
            BackgroundTaskManager.BackgroundTaskStarted += delegate(object sender, EventArgs e)
            {
                PendingTaskCount++;
            };
            BackgroundTaskManager.BackgroundTaskCompleted += delegate(object sender, EventArgs e)
            {
                PendingTaskCount--;
            };
        }

        private void SynchronousTask_Click(object sender, RoutedEventArgs e)
        {
            // Process a task synchronously
            var durationInSeconds = int.Parse((string)(((Button)sender).Tag), CultureInfo.InvariantCulture);
            Log(string.Format(CultureInfo.CurrentCulture, "Started synchronous task taking {0} seconds", durationInSeconds));
            var result = ComputeValue(TimeSpan.FromSeconds(durationInSeconds));
            UseValue(result);
            Log(string.Format(CultureInfo.CurrentCulture, "Finished synchronous task taking {0} seconds", durationInSeconds));
        }

        private void AsynchronousTask_Click(object sender, RoutedEventArgs e)
        {
            // Process a task asynchronously...
            var durationInSeconds = int.Parse((string)(((Button)sender).Tag), CultureInfo.InvariantCulture);
            Log(string.Format(CultureInfo.CurrentCulture, "Started asynchronous task taking {0} seconds", durationInSeconds));

            // ... via BackgroundTaskManager
            BackgroundTaskManager.RunBackgroundTask(
                () =>
                {
                    // Task function runs in the background
                    return ComputeValue(TimeSpan.FromSeconds(durationInSeconds));
                },
                (result) =>
                {
                    // Completion function runs on the UI thread
                    UseValue(result);
                    Log(string.Format(CultureInfo.CurrentCulture, "Finished asynchronous task taking {0} seconds", durationInSeconds));
                });
        }

        private int ComputeValue(TimeSpan delay)
        {
            // Complex synchronous code to compute value goes here...
            Thread.Sleep(delay);
            return 1;
        }

        private void UseValue(int value)
        {
            // Code to use value to update model/UI goes here...
        }

        private void Log(string message)
        {
            // Log a message to the list
            Results.Items.Add(DateTime.Now.ToLongTimeString() + ": " + message);
        }

        // Track the count of pending tasks
        public int PendingTaskCount
        {
            get
            {
                return _pendingTaskCount;
            }
            set
            {
                _pendingTaskCount = value;
                DataContext = _pendingTaskCount;
            }
        }
        private int _pendingTaskCount;
    }
}
